/*
   orxonox - the future of 3D-vertical-scrollers

   Copyright (C) 2004 orx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   ### File Specific:
   main-programmer: Patrick Boenzli
*/

#define DEBUG_SPECIAL_MODULE DEBUG_MODULE_IMPORTER

#include "md3_model.h"

#include "md3_data.h"
#include "md3_mesh.h"
#include "md3_tag.h"
#include "md3_bone_frame.h"

#include "md3_animation_cfg.h"

#include "material.h"

namespace md3
{

  /**
   * md3 model
   */
  MD3Model::MD3Model(std::string filename, float scaling)
  {
    this->md3Data = new MD3Data(filename, filename, scaling);

    MD3AnimationCfg cfg("/home/boenzlip/tmp/q3/Downloads/MOH/q3mdl-alien3/models/players/alien3/animation.cfg");
  }



  MD3Model::~MD3Model()
  {}



  /**
   * this draws the md3 model
   */
  void MD3Model::draw()
  {
    //draw current bone frame
    this->draw(this->md3Data);
  }


  /**
   * draw the md3model
   * @param data: the data to be drawn
   */
  void MD3Model::draw(MD3Data* data)
  {
    if( this->bDrawBones)
    {
      // get bone frame, interpolate if necessary
      if( data->animationState.interpolationFraction != 0.0 &&
          data->animationState.currentFrame != data->animationState.nextFrame) {
        //interpolate bone frame
        this->drawBoneFrame(this->interpolateBoneFrame(data->boneFrames[data->animationState.currentFrame],
                            data->boneFrames[data->animationState.nextFrame],
                            data->animationState.interpolationFraction));
      }
      else {
        //stick with current bone frame
        this->drawBoneFrame(data->boneFrames[data->animationState.currentFrame]);
      }
    }

    //draw all meshes of current frame of this model
    for( int i = 0;  i < data->meshNum; i++) {
      MD3Mesh* mesh = data->meshes[i];

//       gl.glBlendFunc(mesh.GLSrcBlendFunc, mesh.GLDstBlendFunc);
//       gl.glDepthMask(mesh.GLDepthMask);

      if( mesh->header->textureNum > 0 && &mesh->material[0] != NULL)
        mesh->material[0].select();

      // get mesh frame, do interpolation if necessary
      sVec3D* frame;
      if( data->animationState.interpolationFraction != 0.0 &&
          data->animationState.currentFrame != data->animationState.nextFrame) {
        //interpolate mesh frame between the 2 current mesh frames
        frame = this->interpolateMeshFrame(mesh, data->animationState.currentFrame, data->animationState.nextFrame,
                                           data->animationState.interpolationFraction);
      }
      else {
        //no interpolation needed, just draw current frame
        frame = &mesh->meshFrames[data->animationState.currentFrame];
      }

      this->drawMesh(mesh, frame);

      //draw vertex normals
      if( this->bDrawNormals) {
        // get vertex normals, interpolate if necessary
        if( data->animationState.interpolationFraction != 0.0 &&
            data->animationState.currentFrame != data->animationState.nextFrame) {
          //interpolate vertex normals
          this->drawVertexNormals(frame, this->interpolateVertexNormals(&mesh->normals[data->animationState.currentFrame],
                                  &mesh->normals[data->animationState.nextFrame], data->animationState.interpolationFraction));
        }
        else {
          //stick with current vertex normals
          this->drawVertexNormals(frame, &mesh->normals[data->animationState.currentFrame]);
        }
      }
    }

    //draw all models linked to this model

    std::map<int, MD3Data*>::iterator it = data->sortedMap.begin();
    while( it != data->sortedMap.end()) {
      MD3Data* child = it->second;

      //build transformation array m from matrix, interpolate if necessary
      float* m = new float[16];

      MD3Tag* currFrameTag = data->boneFrames[data->animationState.currentFrame]->tags[child->parentTagIndex];

      if( data->animationState.interpolationFraction != 0.0 &&
          data->animationState.currentFrame != data->animationState.nextFrame) {
        //we need to interpolate
        MD3Tag* nextFrameTag = data->boneFrames[data->animationState.nextFrame]->tags[child->parentTagIndex];
        m = this->interpolateTransformation(currFrameTag, nextFrameTag, data->animationState.interpolationFraction);
      }
      else {
        //no interpolation needed, stay with last transformation
        //OpenGL matrix is in column-major order
        m[0] = currFrameTag->matrix[0][0];
        m[1] = currFrameTag->matrix[1][0];
        m[2] = currFrameTag->matrix[2][0];
        m[3] = 0.0f;
        m[4] = currFrameTag->matrix[0][1];
        m[5] = currFrameTag->matrix[1][1];
        m[6] = currFrameTag->matrix[2][1];
        m[7] = 0.0f;
        m[8] = currFrameTag->matrix[0][2];
        m[9] = currFrameTag->matrix[1][2];
        m[10]= currFrameTag->matrix[2][2];
        m[11]= 0.0f;
        m[12] = currFrameTag->position.x;
        m[13] = currFrameTag->position.y;
        m[14] = currFrameTag->position.z;
        m[15] = 1.0f;
      }
    }
  }


  /**
   * draws the mesh
   */
  void MD3Model::drawMesh(MD3Mesh* mesh, sVec3D* frame)
  {}


  /**
   *  drawo vertex normals
   */
  void MD3Model::drawVertexNormals(sVec3D* frame, MD3Normal* normals)
  {}


  /**
   *  interpolate bone frame
   */
  MD3BoneFrame* MD3Model::interpolateBoneFrame(MD3BoneFrame* currBoneFrame, MD3BoneFrame* nextBoneFrame, float frac)
  {}


  /**
   * draw bone frame
   */
  void MD3Model::drawBoneFrame(MD3BoneFrame* frame)
  {}


  /**
   * interpolate mesh frame
   */
  sVec3D* MD3Model::interpolateMeshFrame(MD3Mesh* mesh, int startFrame, int nextMeshFrame, float frac)
  {}


  /**
   * interpolate vertex normal
   */
  MD3Normal* MD3Model::interpolateVertexNormals(MD3Normal* curNormals, MD3Normal* nextNormals, float frac)
  {}


  /**
   * interpolate transformation
   */
  float* MD3Model::interpolateTransformation(MD3Tag* currFrameTag, MD3Tag* nextFrameTag, float frac)
  {}


}
